@@ -1,5 +1,5 @@
 #
-# $Id: Headers.pm,v 1.41 2001/04/12 06:50:28 gisle Exp $
+# $Id: Headers.pm,v 1.24 1996/07/22 13:05:56 aas Exp $
 
 package HTTP::Headers;
 
@@ -10,7 +10,7 @@ HTTP::Headers - Class encapsulating HTTP Message headers
 =head1 SYNOPSIS
 
  require HTTP::Headers;
- $h = new HTTP::Headers;
+ $request = new HTTP::Headers;
 
 =head1 DESCRIPTION
 
@@ -22,17 +22,12 @@ Instances of this class are usually created as member variables of the
 C<HTTP::Request> and C<HTTP::Response> classes, internal to the
 library.
 
-The following methods are available:
-
-=over 4
+=head1 METHODS
 
 =cut
 
-use strict;
-use vars qw($VERSION $TRANSLATE_UNDERSCORE);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.41 $ =~ /(\d+)\.(\d+)/);
 
-use Carp ();
+require Carp;
 
 # Could not use the AutoLoader becase several of the method names are
 # not unique in the first 8 characters.
@@ -44,22 +39,25 @@ use Carp ();
 #    - Request-Headers
 #    - Response-Headers
 #    - Entity-Headers
-# (From draft-ietf-http-v11-spec-rev-01, Nov 21, 1997)
+#    - Aditional Headers (19.6.2)
+# (From draft-ietf-http-v11-spec-06, Jul 4, 1996)
 
 my @header_order = qw(
-   Cache-Control Connection Date Pragma Transfer-Encoding Upgrade Trailer Via
+   Cache-Control Connection Date Pragma Transfer-Encoding Upgrade Via
 
    Accept Accept-Charset Accept-Encoding Accept-Language
-   Authorization Expect From Host
+   Authorization From Host
    If-Modified-Since If-Match If-None-Match If-Range If-Unmodified-Since
-   Max-Forwards Proxy-Authorization Range Referer TE User-Agent
+   Max-Forwards Proxy-Authorization Range Referer User-Agent
 
-   Accept-Ranges Age Location Proxy-Authenticate Retry-After Server Vary
+   Age Location Proxy-Authenticate Public Retry-After Server Vary
    Warning WWW-Authenticate
 
    Allow Content-Base Content-Encoding Content-Language Content-Length
    Content-Location Content-MD5 Content-Range Content-Type
    ETag Expires Last-Modified
+
+   Alternates Content-Version Derived-From Link URI
 );
 
 # Make alternative representations of @header_order.  This is used
@@ -69,15 +67,13 @@ my %header_order;
 my %standard_case;
 for (@header_order) {
     my $lc = lc $_;
-    $header_order{$lc} = ++$i;
+    $header_order{$lc} = $i++;
     $standard_case{$lc} = $_;
 }
 
-$TRANSLATE_UNDERSCORE = 1 unless defined $TRANSLATE_UNDERSCORE;
 
 
-
-=item $h = new HTTP::Headers
+=head2 $h = new HTTP::Headers
 
 Constructs a new C<HTTP::Headers> object.  You might pass some initial
 attribute-value pairs as parameters to the constructor.  I<E.g.>:
@@ -85,37 +81,37 @@ attribute-value pairs as parameters to the constructor.  I<E.g.>:
  $h = new HTTP::Headers
      Date         => 'Thu, 03 Feb 1994 00:00:00 GMT',
      Content_Type => 'text/html; version=3.2',
-     Content_Base => 'http://www.perl.org/';
+     Content_Base => 'http://www.sn.no/';
 
 =cut
 
 sub new
 {
     my($class) = shift;
-    my $self = bless {}, $class;
+    my $self = bless {
+	'_header'   => { },
+    }, $class;
+
     $self->header(@_); # set up initial headers
     $self;
 }
 
 
-=item $h->header($field [=> $value],...)
+=head2 $h->header($field [=> $val],...)
 
 Get or set the value of a header.  The header field name is not case
 sensitive.  To make the life easier for perl users who wants to avoid
 quoting before the => operator, you can use '_' as a synonym for '-'
-in header names (this behaviour can be suppressed by setting
-$HTTP::Headers::TRANSLATE_UNDERSCORE to a FALSE value).
+in header names.
 
-The header() method accepts multiple ($field => $value) pairs, so you
-can update several fields with a single invocation.
+The value argument may be a scalar or a reference to a list of
+scalars. If the value argument is not defined, then the header is not
+modified.
 
-The optional $value argument may be a scalar or a reference to a list
-of scalars. If the $value argument is undefined or not given, then the
-header is not modified.
+The header() method accepts multiple ($field => $value) pairs.
 
-The old value of the last of the $field values is returned.
-Multi-valued fields will be concatenated with "," as separator in
-scalar context.
+The list of previous values for the last $field is returned.  Only the
+first header value is returned in scalar context.
 
  $header->header(MIME_Version => '1.0',
 		 User_Agent   => 'My-Web-Client/0.01');
@@ -128,71 +124,22 @@ scalar context.
 sub header
 {
     my $self = shift;
-    my(@old);
-    while (my($field, $val) = splice(@_, 0, 2)) {
+    my($field, $val, @old);
+    while (($field, $val) = splice(@_, 0, 2)) {
 	@old = $self->_header($field, $val);
     }
-    return @old if wantarray;
-    return $old[0] if @old <= 1;
-    join(", ", @old);
+    wantarray ? @old : $old[0];
 }
 
-
-=item $h->push_header($field, $val)
-
-Add a new field value of the specified header.  The header field name
-is not case sensitive.  The field need not already have a
-value. Previous values for the same field are retained.  The argument
-may be a scalar or a reference to a list of scalars.
-
- $header->push_header(Accept => 'image/jpeg');
-
-=cut
-
-sub push_header
-{
-    Carp::croak('Usage: $h->push_header($field, $val)') if @_ != 3;
-    shift->_header(@_, 'PUSH');
-}
-
-=item $h->init_header($field, $val)
-
-Set the specified header to the given value unless it already has a
-value.  The header field name is not case sensitive.
-
-=cut
-
-sub init_header
-{
-    Carp::croak('Usage: $h->init_header($field, $val)') if @_ != 3;
-    shift->_header(@_, 'INIT');
-}
-
-
-=item $h->remove_header($field,...)
-
-This function removes the headers with the specified names.
-
-=cut
-
-sub remove_header
-{
-    my($self, @fields) = @_;
-    my $field;
-    foreach $field (@fields) {
-	$field =~ tr/_/-/ if $TRANSLATE_UNDERSCORE;
-	delete $self->{lc $field};
-    }
-}
-
-
 sub _header
 {
-    my($self, $field, $val, $op) = @_;
-    $field =~ tr/_/-/ if $TRANSLATE_UNDERSCORE;
+    my($self, $field, $val, $push) = @_;
+    $field =~ tr/_/-/;  # allow use of '_' as alternative to '-' in fields
 
     # $push is only used interally sub push_header
-    Carp::croak('Need a field name') unless length($field);
+
+    Carp::croak('Need a field name') unless defined $field;
+    Carp::croak('Too many parameters') if @_ > 4;
 
     my $lc_field = lc $field;
     unless(defined $standard_case{$lc_field}) {
@@ -201,21 +148,23 @@ sub _header
 	$standard_case{$lc_field} = $field;
     }
 
-    my $h = $self->{$lc_field};
-    my @old = ref($h) ? @$h : (defined($h) ? ($h) : ());
+    my $this_header = \@{$self->{'_header'}{$lc_field}};
 
-    $op ||= "";
-    $val = undef if $op eq 'INIT' && @old;
-    if (defined($val)) {
-	my @new = ($op eq 'PUSH') ? @old : ();
+    my @old = ();
+    if (!$push && defined $this_header) {
+	@old = @$this_header;  # save it so we can return it
+    }
+    if (defined $val) {
+	@$this_header = () unless $push;
 	if (!ref($val)) {
-	    push(@new, $val);
+	    # scalar: create list with single value
+	    push(@$this_header, $val);
 	} elsif (ref($val) eq 'ARRAY') {
-	    push(@new, @$val);
+	    # list: copy list
+	    push(@$this_header, @$val);
 	} else {
 	    Carp::croak("Unexpected field value $val");
 	}
-	$self->{$lc_field} = @new > 1 ? \@new : $new[0];
     }
     @old;
 }
@@ -225,11 +174,17 @@ sub _header
 # recommended "Good Practice" order.
 sub _header_cmp
 {
-    ($header_order{$a} || 999) <=> ($header_order{$b} || 999) || $a cmp $b;
+    # Unknown headers are assign a large value so that they are
+    # sorted last.  This also helps avoiding a warning from -w
+    # about comparing undefined values.
+    $header_order{$a} = 999 unless defined $header_order{$a};
+    $header_order{$b} = 999 unless defined $header_order{$b};
+
+    $header_order{$a} <=> $header_order{$b} || $a cmp $b;
 }
 
 
-=item $h->scan(\&doit)
+=head2 $h->scan(\&doit)
 
 Apply a subroutine to each header in turn.  The callback routine is
 called with two parameters; the name of the field and a single value.
@@ -243,33 +198,29 @@ recommended "Good Practice" order.
 sub scan
 {
     my($self, $sub) = @_;
-    my $key;
-    foreach $key (sort _header_cmp keys %$self) {
-        next if $key =~ /^_/;
-	my $vals = $self->{$key};
-	if (ref($vals)) {
+    my $field;
+    foreach $field (sort _header_cmp keys %{$self->{'_header'}} ) {
+	my $list = $self->{'_header'}{$field};
+	if (defined $list) {
 	    my $val;
-	    for $val (@$vals) {
-		&$sub($standard_case{$key} || $key, $val);
+	    for $val (@$list) {
+		&$sub($standard_case{$field} || $field, $val);
 	    }
-	} else {
-	    &$sub($standard_case{$key} || $key, $vals);
 	}
     }
 }
 
 
-=item $h->as_string([$endl])
+=head2 $h->as_string([$endl])
 
 Return the header fields as a formatted MIME header.  Since it
 internally uses the C<scan()> method to build the string, the result
 will use case as suggested by HTTP Spec, and it will follow
 recommended "Good Practice" of ordering the header fieds.  Long header
-values are not folded. 
+values are I<never> folded.
 
 The optional parameter specifies the line ending sequence to use.  The
-default is C<"\n">.  Embedded "\n" characters in the header will be
-substitued with this line ending sequence.
+default is C<"\n">.
 
 =cut
 
@@ -281,13 +232,8 @@ sub as_string
     my @result = ();
     $self->scan(sub {
 	my($field, $val) = @_;
-	if ($val =~ /\n/) {
-	    # must handle header values with embedded newlines with care
-	    $val =~ s/\s+$//;          # trailing newlines and space must go
-	    $val =~ s/\n\n+/\n/g;      # no empty lines
-	    $val =~ s/\n([^\040\t])/\n $1/g;  # intial space for continuation
-	    $val =~ s/\n/$endl/g;      # substitute with requested line ending
-	}
+	Carp::croak("HTTP header ($field) contains newline")
+	   if $val =~ /\n/;
 	push(@result, "$field: $val");
     });
 
@@ -295,23 +241,29 @@ sub as_string
 }
 
 
-=item $h->clone
+# The remaining functions should autoloaded only when needed
+
+# A bug in 5.002gamma makes it risky to have POD text inside the
+# autoloaded section of the code, so we keep the documentation before
+# the __DATA__ token.
+
+=head2 $h->push_header($field, $val)
+
+Add a new field value of the specified header.  The header field name
+is not case sensitive.  The field need not already have a
+value. Previous values for the same field are retained.  The argument
+may be a scalar or a reference to a list of scalars.
+
+ $header->push_header(Accept => 'image/jpeg');
+
+=head2 $h->remove_header($field,...)
+
+This function removes the headers with the specified names.
+
+=head2 $h->clone
 
 Returns a copy of this HTTP::Headers object.
 
-=back
-
-=cut
-
-sub clone
-{
-    my $self = shift;
-    my $clone = new HTTP::Headers;
-    $self->scan(sub { $clone->push_header(@_);} );
-    $clone;
-}
-
-
 =head1 CONVENIENCE METHODS
 
 The most frequently used headers can also be accessed through the
@@ -323,30 +275,26 @@ Methods that deal with dates/times always convert their value to system
 time (seconds since Jan 1, 1970) and they also expect this kind of
 value when the header value is set.
 
-=over 4
-
-=item $h->date
+=head2 $h->date
 
 This header represents the date and time at which the message was
 originated. I<E.g.>:
 
   $h->date(time);  # set current date
 
-=item $h->expires
+=head2 $h->expires
 
 This header gives the date and time after which the entity should be
 considered stale.
 
-=item $h->if_modified_since
-
-=item $h->if_unmodified_since
+=head2 $h->if_modified_since
 
 This header is used to make a request conditional.  If the requested
-resource has (not) been modified since the time specified in this field,
+resource has not been modified since the time specified in this field,
 then the server will return a C<"304 Not Modified"> response instead of
 the document itself.
 
-=item $h->last_modified
+=head2 $h->last_modified
 
 This header indicates the date and time at which the resource was last
 modified. I<E.g.>:
@@ -356,7 +304,7 @@ modified. I<E.g.>:
 	...
   }
 
-=item $h->content_type
+=head2 $h->content_type
 
 The Content-Type header field indicates the media type of the message
 content. I<E.g.>:
@@ -373,73 +321,60 @@ an array context.  This makes it safe to do the following:
      ...
   }
 
-=item $h->content_encoding
+=head2 $h->content_encoding
 
 The Content-Encoding header field is used as a modifier to the
 media type.  When present, its value indicates what additional
 encoding mechanism has been applied to the resource.
 
-=item $h->content_length
+=head2 $h->content_length
 
 A decimal number indicating the size in bytes of the message content.
 
-=item $h->content_language
-
-The natural language(s) of the intended audience for the message
-content.  The value is one or more language tags as defined by RFC
-1766.  Eg. "no" for Norwegian and "en-US" for US-English.
-
-=item $h->title
+=head2 $h->title
 
 The title of the document.  In libwww-perl this header will be
 initialized automatically from the E<lt>TITLE>...E<lt>/TITLE> element
 of HTML documents.  I<This header is no longer part of the HTTP
 standard.>
 
-=item $h->user_agent
+=head2 $h->user_agent
 
 This header field is used in request messages and contains information
 about the user agent originating the request.  I<E.g.>:
 
   $h->user_agent('Mozilla/1.2');
 
-=item $h->server
+=head2 $h->server
 
 The server header field contains information about the software being
 used by the originating server program handling the request.
 
-=item $h->from
+=head2 $h->from
 
 This header should contain an Internet e-mail address for the human
 user who controls the requesting user agent.  The address should be
 machine-usable, as defined by RFC822.  E.g.:
 
-  $h->from('King Kong <king@kong.com>');
+  $h->from('Gisle Aas <aas@sn.no>');
 
-=item $h->referer
+=head2 $h->referer
 
 Used to specify the address (URI) of the document from which the
 requested resouce address was obtained.
 
-=item $h->www_authenticate
+=head2 $h->www_authenticate
 
 This header must be included as part of a "401 Unauthorized" response.
 The field value consist of a challenge that indicates the
 authentication scheme and parameters applicable to the requested URI.
 
-=item $h->proxy_authenticate
+=head2 $h->authorization
 
-This header must be included in a "407 Proxy Authentication Required"
-response.
+A user agent that wishes to authenticate itself with a server, may do
+so by including this header.
 
-=item $h->authorization
-
-=item $h->proxy_authorization
-
-A user agent that wishes to authenticate itself with a server or a
-proxy, may do so by including these headers.
-
-=item $h->authorization_basic
+=head2 $h->authorization_basic
 
 This method is used to get or set an authorization header that use the
 "Basic Authentication Scheme".  In array context it will return two
@@ -450,17 +385,39 @@ When used to set the header value, it expects two arguments.  I<E.g.>:
 
   $h->authorization_basic($uname, $password);
 
-The method will croak if the $uname contains a colon ':'.
-
-=item $h->proxy_authorization_basic
-
-Same as authorization_basic() but will set the "Proxy-Authorization"
-header instead.
-
-=back
-
 =cut
 
+1;
+
+#__DATA__
+
+sub clone
+{
+    my $self = shift;
+    my $clone = new HTTP::Headers;
+    $self->scan(sub { $clone->push_header(@_);} );
+    $clone;
+}
+
+sub push_header
+{
+    Carp::croak('Usage: $h->push_header($field, $val)') if @_ != 3;
+    shift->_header(@_, 'PUSH');
+}
+
+
+sub remove_header
+{
+    my($self, @fields) = @_;
+    my $field;
+    foreach $field (@fields) {
+	$field =~ tr/_/-/;
+	delete $self->{'_header'}{lc $field};
+    }
+}
+
+# Convenience access functions
+
 sub _date_header
 {
     require HTTP::Date;
@@ -472,33 +429,26 @@ sub _date_header
     HTTP::Date::str2time($old);
 }
 
-sub date                { shift->_date_header('Date',                @_); }
-sub expires             { shift->_date_header('Expires',             @_); }
-sub if_modified_since   { shift->_date_header('If-Modified-Since',   @_); }
-sub if_unmodified_since { shift->_date_header('If-Unmodified-Since', @_); }
-sub last_modified       { shift->_date_header('Last-Modified',       @_); }
-
-# This is used as a private LWP extention.  The Client-Date header is
-# added as a timestamp to a response when it has been received.
-sub client_date         { shift->_date_header('Client-Date',         @_); }
+sub date              { shift->_date_header('Date',              @_); }
+sub expires           { shift->_date_header('Expires',           @_); }
+sub if_modified_since { shift->_date_header('If-Modified-Since', @_); }
+sub last_modified     { shift->_date_header('Last-Modified',     @_); }
 
 # The retry_after field is dual format (can also be a expressed as
 # number of seconds from now), so we don't provide an easy way to
 # access it until we have know how both these interfaces can be
-# addressed.  One possibility is to return a negative value for
-# relative seconds and a positive value for epoch based time values.
+# addressed.
 #sub retry_after       { shift->_date_header('Retry-After',       @_); }
 
 sub content_type      {
   my $ct = (shift->_header('Content-Type', @_))[0];
-  return '' unless defined($ct) && length($ct);
+  return '' unless defined $ct;
   my @ct = split(/\s*;\s*/, lc($ct));
   wantarray ? @ct : $ct[0];
 }
 
 sub title             { (shift->_header('Title',            @_))[0] }
 sub content_encoding  { (shift->_header('Content-Encoding', @_))[0] }
-sub content_language  { (shift->_header('Content-Language', @_))[0] }
 sub content_length    { (shift->_header('Content-Length',   @_))[0] }
 
 sub user_agent        { (shift->_header('User-Agent',       @_))[0] }
@@ -506,44 +456,25 @@ sub server            { (shift->_header('Server',           @_))[0] }
 
 sub from              { (shift->_header('From',             @_))[0] }
 sub referer           { (shift->_header('Referer',          @_))[0] }
-sub warning           { (shift->_header('Warning',          @_))[0] }
-*referrer = \&referer;  # on tchrist's request
 
 sub www_authenticate  { (shift->_header('WWW-Authenticate', @_))[0] }
 sub authorization     { (shift->_header('Authorization',    @_))[0] }
 
-sub proxy_authenticate  { (shift->_header('Proxy-Authenticate',  @_))[0] }
-sub proxy_authorization { (shift->_header('Proxy-Authorization', @_))[0] }
-
-sub authorization_basic       { shift->_basic_auth("Authorization",       @_) }
-sub proxy_authorization_basic { shift->_basic_auth("Proxy-Authorization", @_) }
-
-sub _basic_auth {
+sub authorization_basic {
     require MIME::Base64;
-    my($self, $h, $user, $passwd) = @_;
-    my($old) = $self->_header($h);
+    my($self, $user, $passwd) = @_;
+    my($old) = $self->_header('Authorization');
     if (defined $user) {
-	Carp::croak("Basic authorization user name can't contain ':'")
-	  if $user =~ /:/;
 	$passwd = '' unless defined $passwd;
-	$self->_header($h => 'Basic ' .
-                             MIME::Base64::encode("$user:$passwd", ''));
+	$self->_header('Authorization',
+		       'Basic ' . MIME::Base64::encode("$user:$passwd", ''));
     }
     if (defined $old && $old =~ s/^\s*Basic\s+//) {
 	my $val = MIME::Base64::decode($old);
 	return $val unless wantarray;
 	return split(/:/, $val, 2);
     }
-    return;
+    undef;
 }
 
-=head1 COPYRIGHT
-
-Copyright 1995-2001 Gisle Aas.
-
-This library is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself.
-
-=cut
-
 1;
