@@ -1,12 +1,16 @@
 #
-# $Id: LWP.pm,v 1.109 2001/09/17 19:25:17 gisle Exp $
+# $Id: LWP.pm,v 1.35 1996/06/24 16:46:07 aas Exp $
 
 package LWP;
 
-$VERSION = "5.5397";
+require Exporter;
+@ISA=qw(Exporter);  # for version number checking
+
+$VERSION = "5.01";
+
 sub Version { $VERSION; }
 
-require 5.004;
+require 5.002;
 require LWP::UserAgent;  # this should load everything you need
 
 1;
@@ -17,22 +21,17 @@ __END__
 
 LWP - Library for WWW access in Perl
 
-=head1 SYNOPSIS
-
-  use LWP;
-  print "This is libwww-perl-$LWP::VERSION\n";
-
 
 =head1 DESCRIPTION
 
 Libwww-perl is a collection of Perl modules which provides a simple
-and consistent application programming interface (API) to the World-Wide Web.  The
+and consistent programming interface (API) to the World-Wide Web.  The
 main focus of the library is to provide classes and functions that
-allow you to write WWW clients, thus libwww-perl is a WWW
+allow you to write WWW clients, thus libwww-perl said to be a WWW
 client library. The library also contain modules that are of more
 general use.
 
-Most modules in this library are object oriented.  The user
+The main architecture of the library is object oriented.  The user
 agent, requests sent and responses received from the WWW server are
 all represented by objects.  This makes a simple and powerful
 interface to these services.  The interface should be easy to extend
@@ -44,27 +43,27 @@ The main features of the library are:
 
 =item *
 
-Contains various reusable components (modules) that can be
+Contains various reuseable components (modules) that can be
 used separately or together.
 
 =item *
 
 Provides an object oriented model of HTTP-style communication.  Within
-this framework we currently support access to http, https, gopher, ftp, news,
+this framework we currently support access to http, gopher, ftp, news,
 file, and mailto resources.
 
 =item *
 
-Provides a full object oriented interface or
-a very simple procedural interface.
+The library be used through the full object oriented interface or
+through a very simple procedural interface.
 
 =item *
 
-Supports the basic and digest authorization schemes.
+Support the basic and digest authorization schemes.
 
 =item *
 
-Supports transparent redirect handling.
+Transparent redirect handling.
 
 =item *
 
@@ -72,25 +71,30 @@ Supports access through proxy servers.
 
 =item *
 
-Provides parser for F<robots.txt> files and a framework for constructing robots.
+URL handling (both absolute and relative URLs are supported).
 
 =item *
 
-Cooperates with Tk.  A simple Tk-based GUI browser
-called 'tkweb' is distributed with the Tk extension for perl.
+A parser for F<robots.txt> files and a framework for constructing robots.
 
 =item *
 
-Implements HTTP content negotiation algorithm that can
+An experimental HTML parser and formatters (for PostScript and plain
+text).
+
+=item *
+
+The library can cooperate with Tk.  A simple Tk-based GUI browser
+called 'tkweb' is distributed with the Tk extention for perl.
+
+=item *
+
+An implementation of the HTTP content negotiation algorithm that can
 be used both in protocol modules and in server scripts (like CGI
 scripts).
 
 =item *
 
-Supports HTTP cookies.
-
-=item *
-
 A simple command line client application called C<lwp-request>.
 
 =back
@@ -100,14 +104,14 @@ A simple command line client application called C<lwp-request>.
 
 
 The libwww-perl library is based on HTTP style communication. This
-section tries to describe what that means.
+section try to describe what that means.
 
 Let us start with this quote from the HTTP specification document
 <URL:http://www.w3.org/pub/WWW/Protocols/>:
 
 =over 3
 
-=item
+=item *
 
 The HTTP protocol is based on a request/response paradigm. A client
 establishes a connection with a server and sends a request to the
@@ -116,11 +120,11 @@ followed by a MIME-like message containing request modifiers, client
 information, and possible body content. The server responds with a
 status line, including the message's protocol version and a success or
 error code, followed by a MIME-like message containing server
-information, entity meta-information, and possible body content.
+information, entity metainformation, and possible body content.
 
 =back
 
-What this means to libwww-perl is that communication always take place
+What this means to libwww-perl is that communcation always take place
 through these steps: First a I<request> object is created and
 configured. This object is then passed to a server and we get a
 I<response> object in return that we can examine. A request is always
@@ -136,16 +140,16 @@ parameters and the response will contain the query result.  If we want
 to send a mail message to somebody then we send a request object which
 contains our message to the mail server and the response object will
 contain an acknowledgment that tells us that the message has been
-accepted and will be forwarded to the recipient(s).
+accepted and will be forwarded to the receipient(s).
 
 It is as simple as that!
 
 
 =head2 The Request Object
 
-The libwww-perl request object has the class name C<HTTP::Request>.
-The fact that the class name uses C<HTTP::> as a
-prefix only implies that we use the HTTP model of communication.  It
+The request object has the class name C<HTTP::Request> in
+libwww-perl. The fact that the class name use C<HTTP::> as a name
+prefix only implies that we use the HTTP model of communication. It
 does not limit the kind of services we can try to pass this I<request>
 to.  For instance, we will send C<HTTP::Request>s both to ftp and
 gopher servers, as well as to the local file system.
@@ -163,14 +167,14 @@ B<POST> and B<HEAD>.
 =item *
 
 The B<url> is a string denoting the protocol, server and
-the name of the "document" we want to access.  The B<url> might
+the name of the "document" we want to access.  The url might
 also encode various other parameters.
 
 =item *
 
 The B<headers> contain additional information about the
 request and can also used to describe the content.  The headers
-are a set of keyword/value pairs.
+is a set of keyword/value pairs.
 
 =item *
 
@@ -180,25 +184,25 @@ The B<content> is an arbitrary amount of data.
 
 =head2 The Response Object
 
-The libwww-perl response object has the class name C<HTTP::Response>.
-The main attributes of objects of this class are:
+The request object has the class name C<HTTP::Response> in
+libwww-perl.  The main attributes of objects of this class are:
 
 =over 3
 
 =item *
 
-The B<code> is a numerical value that indicates the overall
+The B<code> is a numerical value that encode the overall
 outcome of the request.
 
 =item *
 
-The B<message> is a short, human readable string that
+The B<message> is a short (human readable) string that
 corresponds to the I<code>.
 
 =item *
 
 The B<headers> contain additional information about the
-response and describe the content.
+response and they also describe the content.
 
 =item *
 
@@ -207,7 +211,7 @@ The B<content> is an arbitrary amount of data.
 =back
 
 Since we don't want to handle all possible I<code> values directly in
-our programs, a libwww-perl response object has methods that can be
+our programs, the libwww-perl response object have methods that can be
 used to query what kind of response this is.  The most commonly used
 response classification methods are:
 
@@ -220,7 +224,7 @@ The request was was successfully received, understood or accepted.
 =item is_error()
 
 The request failed.  The server or the resource might not be
-available, access to the resource might be denied or other things might
+available, access to the resouce might be denied or other things might
 have failed for some reason.
 
 =back
@@ -230,68 +234,72 @@ have failed for some reason.
 Let us assume that we have created a I<request> object. What do we
 actually do with it in order to receive a I<response>?
 
-The answer is that you pass it to a I<user agent> object and this
-object takes care of all the things that need to be done
-(like low-level communication and error handling) and returns
-a I<response> object. The user agent represents your
-application on the network and provides you with an interface that
-can accept I<requests> and return I<responses>.
+The answer is that you pass it on to a I<user agent> object and this
+object will take care of all the things that need to be done
+(low-level communcation and error handling). The user agent will give
+you back a I<response> object. The user agent represents your
+application on the network and it provides you with an interface that
+can accept I<requests> and will return I<responses>.
 
-The user agent is an interface layer between
+You should think about the user agent as an interface layer between
 your application code and the network.  Through this interface you are
 able to access the various servers on the network.
 
 The libwww-perl class name for the user agent is
-C<LWP::UserAgent>.  Every libwww-perl application that wants to
-communicate should create at least one object of this class. The main
+C<LWP::UserAgent>. Every libwww-perl application that wants to
+communicate should create at least one object of this kind. The main
 method provided by this object is request(). This method takes an
-C<HTTP::Request> object as argument and (eventually) returns a
+C<HTTP::Request> object as argument and will (eventually) return a
 C<HTTP::Response> object.
 
-The user agent has many other attributes that let you
+The user agent has many other attributes that lets you
 configure how it will interact with the network and with your
-application.
+application code.
 
 =over 3
 
 =item *
 
-The B<timeout> specifies how much time we give remote servers to
-respond before the library disconnects and creates an
+The B<timeout> specify how much time we give remote servers in
+creating responses before the library disconnect and creates an
 internal I<timeout> response.
 
 =item *
 
-The B<agent> specifies the name that your application should use when it
+The B<agent> specify the name that your application should use when it
 presents itself on the network.
 
 =item *
 
 The B<from> attribute can be set to the e-mail address of the person
-responsible for running the application.  If this is set, then the
+reponsible for running the application.  If this is set, then the
 address will be sent to the servers with every request.
 
 =item *
 
-The B<parse_head> specifies whether we should initialize response
-headers from the E<lt>head> section of HTML documents.
+The B<use_alarm> specify if it is ok for the user agent to use the
+alarm(2) system to implement timeouts.
 
 =item *
 
-The B<proxy> and B<no_proxy> attributes specify if and when to go through
-a proxy server. <URL:http://www.w3.org/pub/WWW/Proxies/>
+The B<use_eval> specify if the agent should raise an
+expection (C<die> in perl) if an error condition occur.
 
 =item *
 
-The B<credentials> provide a way to set up user names and
-passwords needed to access certain services.
+The B<proxy> and B<no_proxy> specify if and when communication should
+go through a proxy server. <URL:http://www.w3.org/pub/WWW/Proxies/>
+
+=item *
+
+The B<credentials> provide a way to set up usernames and
+passwords that is needed to access certain services.
 
 =back
 
-Many applications want even more control over how they interact
-with the network and they get this by sub-classing
-C<LWP::UserAgent>.  The library includes a
-sub-class, C<LWP::RobotUA>, for robot applications.
+Many applications would want even more control over how they
+interact with the network and they get this by specializing the
+C<LWP::UserAgent> by sub-classing.
 
 =head2 An Example
 
@@ -301,7 +309,7 @@ represented in actual perl code:
   # Create a user agent object
   use LWP::UserAgent;
   $ua = new LWP::UserAgent;
-  $ua->agent("AgentName/0.1 " . $ua->agent);
+  $ua->agent("AgentName/0.1");
 
   # Create a request
   my $req = new HTTP::Request POST => 'http://www.perl.com/cgi-bin/BugGlimpse';
@@ -318,32 +326,27 @@ represented in actual perl code:
       print "Bad luck this time\n";
   }
 
-The $ua is created once when the application starts up.  New request
-objects should normally created for each request sent.
-
 
 =head1 NETWORK SUPPORT
 
-This section discusses the various protocol schemes and
-the HTTP style methods that headers may be used for each.
+This section goes through the various protocol schemes and describe
+the HTTP style methods that are supported and the headers that might
+have any effect.
 
 For all requests, a "User-Agent" header is added and initialized from
-the $ua->agent attribute before the request is handed to the network
+the $ua->agent value before the request is handed to the network
 layer.  In the same way, a "From" header is initialized from the
-$ua->from attribute.
+$ua->from value.
 
-For all responses, the library adds a header called "Client-Date".
-This header holds the time when the response was received by
-your application.  The format and semantics of the header are the
-same as the server created "Date" header.  You may also encounter other
-"Client-XXX" headers.  They are all generated by the library
-internally and are not received from the servers.
+For all responses, the library will add a header called "Client-Date".
+This format and semantics of the header is just like the server
+created "Date" header.
 
 =head2 HTTP Requests
 
-HTTP requests are just handed off to an HTTP server and it
-decides what happens.  Few servers implement methods beside the usual
-"GET", "HEAD", "POST" and "PUT", but CGI-scripts may implement
+HTTP request are really just handed off to an HTTP server and it will
+decide what happens.  Few servers implement methods beside the usual
+"GET", "HEAD", "POST" and "PUT" but CGI-scripts can really implement
 any method they like.
 
 If the server is not available then the library will generate an
@@ -352,97 +355,79 @@ internal error response.
 The library automatically adds a "Host" and a "Content-Length" header
 to the HTTP request before it is sent over the network.
 
-For GET request you might want to add the "If-Modified-Since" header
+For GET request you migth want to add the "If-Modified-Since" header
 to make the request conditional.
 
 For POST request you should add the "Content-Type" header.  When you
-try to emulate HTML E<lt>FORM> handling you should usually let the value
+try to emulate HTML <FORM> handling you should usually let the value
 of the "Content-Type" header be "application/x-www-form-urlencoded".
 See L<lwpcook> for examples of this.
 
 The libwww-perl HTTP implementation currently support the HTTP/1.0
 protocol.  HTTP/0.9 servers are also handled correctly.
 
-The library allows you to access proxy server through HTTP.  This
-means that you can set up the library to forward all types of request
-through the HTTP protocol module.  See L<LWP::UserAgent> for
-documentation of this.
+The library allows you to access proxy server through HTTP.  You can
+set up the library to forward all types of request through the HTTP
+protocol module.  See L<LWP::UserAgent>
 
 
-=head2 HTTPS Requests
-
-HTTPS requests are HTTP requests over an encrypted network connection
-using the SSL protocol developed by Netscape.  Everything about HTTP
-requests above also apply to HTTPS requests.  In addition the library
-will add the headers "Client-SSL-Cipher", "Client-SSL-Cert-Subject" and
-"Client-SSL-Cert-Issuer" to the response.  These headers denote the
-encryption method used and the name of the server owner.
-
-The request can contain the header "If-SSL-Cert-Subject" in order to
-make the request conditional on the content of the server certificate.
-If the certificate subject does not match, no request is sent to the
-server and an internally generated error response is returned.  The
-value of the "If-SSL-Cert-Subject" header is interpreted as a Perl
-regular expression.
 
 
 =head2 FTP Requests
 
-The library currently supports GET, HEAD and PUT requests.  GET
-retrieves a file or a directory listing from an FTP server.  PUT
-stores a file on a ftp server.
+The library currently support GET, HEAD and PUT requests.  GET will
+retrive a file or a directory listing from an FTP server.  PUT will
+store a file on a ftp server.
 
 You can specify a ftp account for servers that want this in addition
-to user name and password.  This is specified by including an "Account"
+username and password.  This is specified by passing an "Account"
 header in the request.
 
-User name/password can be specified using basic authorization or be
-encoded in the URL.  Failed logins return an UNAUTHORIZED response with
-"WWW-Authenticate: Basic" and can be treated like basic authorization
+Username/password can be specified using basic authorization or be
+encoded in the URL.  Bad logins return an UNAUTHORIZED response with
+"WWW-Authenticate: Basic" and can be treated as basic authorization
 for HTTP.
 
-The library supports ftp ASCII transfer mode by specifying the "type=a"
+The library support ftp ascii transfer mode by specifying the "type=a"
 parameter in the URL.
 
 Directory listings are by default returned unprocessed (as returned
 from the ftp server) with the content media type reported to be
-"text/ftp-dir-listing". The C<File::Listing> module provides methods
+"text/ftp-dir-listing". The C<File::Listing> module provide a function
 for parsing of these directory listing.
 
 The ftp module is also able to convert directory listings to HTML and
 this can be requested via the standard HTTP content negotiation
-mechanisms (add an "Accept: text/html" header in the request if you
-want this).
+mechanisms (add an "Accept: text/html" header in the request).
 
-For normal file retrievals, the "Content-Type" is guessed based on the
-file name suffix. See L<LWP::MediaTypes>.
+The normal file retrievals, the "Content-Type" is guessed based on the
+file name suffix. See L<LWP::MediaTypes>
 
-The "If-Modified-Since" request header works for servers that implement
-the MDTM command.  It will probably not work for directory listings though.
+The "If-Modified-Since" header is not honored yet.
 
 Example:
 
-  $req = HTTP::Request->new(GET => 'ftp://me:passwd@ftp.some.where.com/');
-  $req->header(Accept => "text/html, */*;q=0.1");
+  $req = HTTP::Request->new('GET', 'ftp://me:passwd@ftp.some.where.com/');
+  $req->header("Accept", "text/html, */*;q=0.1");
 
 =head2 News Requests
 
 Access to the USENET News system is implemented through the NNTP
 protocol.  The name of the news server is obtained from the
 NNTP_SERVER environment variable and defaults to "news".  It is not
-possible to specify the hostname of the NNTP server in news: URLs.
+possible to specify the hostname of the NNTP server in the news:-URLs.
 
-The library supports GET and HEAD to retrieve news articles through the
+The library support GET and HEAD to retrive news articles through the
 NNTP protocol.  You can also post articles to newsgroups by using
-(surprise!) the POST method.
+(suprise) the POST method.
 
 GET on newsgroups is not implemented yet.
 
 Examples:
 
-  $req = HTTP::Request->new(GET => 'news:abc1234@a.sn.no');
+  $req = HTTP::Request->new('GET', 'news:abc1234@a.sn.no');
 
-  $req = HTTP::Request->new(POST => 'news:comp.lang.perl.test');
+  $req = HTTP::Request->new('POST', 'news:comp.lang.perl.test');
   $req->header(Subject => 'This is a test',
                From    => 'me@some.where.org');
   $req->content(<<EOT);
@@ -453,18 +438,18 @@ Examples:
 
 =head2 Gopher Request
 
-The library supports the GET and HEAD methods for gopher requests.  All
-request header values are ignored.  HEAD cheats and returns a
-response without even talking to server.
+The library supports the GET and HEAD method for gopher request.  All
+request header values are ignored.  HEAD cheats and will return a
+reponse without even talking to server.
 
-Gopher menus are always converted to HTML.
+Gopher menues are always converted to HTML.
 
 The response "Content-Type" is generated from the document type
 encoded (as the first letter) in the request URL path itself.
 
 Example:
 
-  $req = HTTP::Request->new(GET => 'gopher://gopher.sn.no/');
+  $req = HTTP::Request->new('GET', 'gopher://gopher.sn.no/');
 
 
 
@@ -481,7 +466,7 @@ guessed based on the file suffix.
 
 Example:
 
-  $req = HTTP::Request->new(GET => 'file:/etc/passwd');
+  $req = HTTP::Request->new('GET', 'file:/etc/passwd');
 
 
 =head2 Mailto Request
@@ -492,14 +477,14 @@ The "To" header is initialized from the mail address in the URL.
 
 Example:
 
-  $req = HTTP::Request->new(POST => 'mailto:libwww@perl.org');
-  $req->header(Subject => "subscribe");
-  $req->content("Please subscribe me to the libwww-perl mailing list!\n");
+  $req = HTTP::Request->new('POST', 'mailto:libwww-perl-request@ics.uci.edu');
+  $req->header("Subject", "subscribe");
+  $req->content("Please add me to the libwww-perl mailing list!\n");
 
 
 =head1 OVERVIEW OF CLASSES AND PACKAGES
 
-This table should give you a quick overview of the classes provided by the
+This table should give you a quick overview of the classes used by the
 library. Indentation shows class inheritance.
 
  LWP::MemberMixin   -- Access to member variables of Perl5 classes
@@ -511,17 +496,20 @@ library. Indentation shows class inheritance.
      LWP::Protocol::ftp   -- ftp:// access
      ...
 
- LWP::Authen::Basic -- Handle 401 and 407 responses
- LWP::Authen::Digest
+ LWP::Socket        -- Socket creation and IO
 
  HTTP::Headers      -- MIME/RFC822 style header (used by HTTP::Message)
  HTTP::Message      -- HTTP style message
    HTTP::Request    -- HTTP request
    HTTP::Response   -- HTTP response
- HTTP::Daemon       -- A HTTP server class
+
+ URI::URL           -- Uniform Resource Locators
 
  WWW::RobotRules    -- Parse robots.txt files
-   WWW::RobotRules::AnyDBM_File -- Persistent RobotRules
+
+ HTML::Parser       -- Parse HTML documents
+ HTML::Element      -- Building block for the parser
+ HTML::Formatter    -- Convert HTML to readable formats
 
 The following modules provide various functions and definitions.
 
@@ -534,25 +522,39 @@ The following modules provide various functions and definitions.
  HTTP::Negotiate    -- HTTP content negotiation calculation
  File::Listing      -- Parse directory listings
 
+HTTP use the Base64 encoding at some places.  The QuotedPrint module
+is just included to make the MIME:: collection more complete.
+
+ MIME::Base64       -- Base64 encoding/decoding routines
+ MIME::QuotedPrint  -- Quoted Printanle encoding/decoding routines
+
+The following modules does not have much to do with WWW, but are
+included just because I am lazy and did not bother to make separate
+distributions for them.  Regard them as bonus.
+
+ Font::AFM          -- Parse Adobe Font Metric files
+ File::CounterFile  -- Persistent counter class
+
 
 =head1 MORE DOCUMENTATION
 
 All modules contain detailed information on the interfaces they
-provide.  The I<lwpcook> manpage is the libwww-perl cookbook that contain
+provide.  The L<lwpcook> is the libwww-perl cookbook that contain
 examples of typical usage of the library.  You might want to take a
 look at how the scripts C<lwp-request>, C<lwp-rget> and C<lwp-mirror>
 are implemented.
 
 =head1 BUGS
 
-The library can not handle multiple simultaneous requests yet.  Also,
-check out what's left in the TODO file.
+The library can not handle multiple simultaneous requests.  The HTML::
+modules are still experimental.  Also, check out what's left in the
+TODO file.
 
 =head1 ACKNOWLEDGEMENTS
 
-This package owes a lot in motivation, design, and code, to the
+This package ows a lot in motivation, design, and code, to the
 libwww-perl library for Perl 4, maintained by Roy Fielding
-E<lt>fielding@ics.uci.edu>.
+<fielding@ics.uci.edu>.
 
 That package used work from Alberto Accomazzi, James Casey, Brooks
 Cutter, Martijn Koster, Oscar Nierstrasz, Mel Melchner, Gertjan van
@@ -567,7 +569,7 @@ Koenig, Jared Rhine, and Jack Shirazi.
 
 =head1 COPYRIGHT
 
-  Copyright 1995-2001, Gisle Aas
+  Copyright 1995-1996, Gisle Aas
   Copyright 1995, Martijn Koster
 
 This library is free software; you can redistribute it and/or
@@ -575,12 +577,11 @@ modify it under the same terms as Perl itself.
 
 =head1 AVAILABILITY
 
-The latest version of this library is likely to be available from CPAN
-as well as:
+The latest version of this library is likly to be available from:
 
- http://www.linpro.no/lwp/
+ http://www.sn.no/libwww-perl/
 
-The best place to discuss this code is on the <libwww@perl.org>
-mailing list.
+The best place to discuss this code is on the
+<libwww-perl@ics.uci.edu> mailing list.
 
 =cut
